SystemOrganization addCategory: #'Minecraft-Pi-Base'!

Object subclass: #Minecraft
	instanceVariableNames: 'connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Minecraft-Pi-Base'!

!Minecraft commentStamp: 'bf 2/24/2013 23:30' prior: 0!
Squeak bindings for "Minecraft: Pi Edition"
Author: Bert Freudenberg <bert@freudenbergs.de>
See http://pi.minecraft.net/

This class allows to communicate with a Minecraft game running on a Raspberry Pi. Squeak can either run on the same machine or remotely.

Instance Variables
	connection:		<MinecraftConnection>

Example
	| mc |
	mc := Minecraft connectTo: 'raspberrypi'.
	mc chatPost: 'Hi from Squeak!!'.
	Transcript show: 'Player pos: '; show: mc playerPos; cr.
	mc close.
!

!Minecraft class methodsFor: 'instance creation' stamp: 'bf 2/24/2013 21:18'!
connect
	^self connectTo: 'localhost'! !

!Minecraft class methodsFor: 'instance creation' stamp: 'bf 2/24/2013 21:19'!
connectTo: hostName
	^self connectTo: hostName port: 4711
! !

!Minecraft class methodsFor: 'instance creation' stamp: 'bf 2/24/2013 21:19'!
connectTo: hostName port: portNumber
	^self new connection: (MinecraftConnection host: hostName port: portNumber)
! !

!Minecraft methodsFor: 'blocks' stamp: 'bf 2/25/2013 00:30'!
blockAndDataAt: vector3
	"return block id and data"
	^self toNumbers: (connection send: 'world.getBlockWithData' args: vector3 floor; receive)
! !

!Minecraft methodsFor: 'blocks' stamp: 'bf 2/25/2013 00:54'!
blockAt: vector3
	"return block id"
	^(connection send: 'world.getBlock' args: vector3 floor; receive) asNumber
! !

!Minecraft methodsFor: 'blocks' stamp: 'bf 2/25/2013 00:22'!
blockAt: vector3 put: blockIdOrIdAndData
	connection send: 'world.setBlock' args: vector3 floor, (self toArray: blockIdOrIdAndData).
! !

!Minecraft methodsFor: 'blocks' stamp: 'bf 2/25/2013 00:23'!
blocksFrom: from3 to: to3 put: blockIdOrIdAndData
	"Fill cube with given block id (or block id and data)"
	connection send: 'world.setBlocks' args: from3 floor, to3 floor, (self toArray: blockIdOrIdAndData).
! !

!Minecraft methodsFor: 'camera' stamp: 'bf 2/24/2013 22:16'!
cameraModeFixed
	^connection send: 'camera.mode.setFixed'! !

!Minecraft methodsFor: 'camera' stamp: 'bf 2/24/2013 22:17'!
cameraModeFollow
	^connection send: 'camera.mode.setFollow'! !

!Minecraft methodsFor: 'camera' stamp: 'bf 2/24/2013 22:16'!
cameraModeNormal
	^connection send: 'camera.mode.setNormal'! !

!Minecraft methodsFor: 'camera' stamp: 'bf 2/24/2013 22:02'!
cameraPos: vector3
	^connection send: 'camera.setPos' args: vector3	! !

!Minecraft methodsFor: 'chat' stamp: 'bf 2/24/2013 21:55'!
chatPost: aString
	connection send: 'chat.post' args: aString
! !

!Minecraft methodsFor: 'checkpoints' stamp: 'bf 2/24/2013 23:56'!
checkpointRestore
	connection send: 'world.checkpoint.restore'.

! !

!Minecraft methodsFor: 'checkpoints' stamp: 'bf 2/24/2013 23:56'!
checkpointSave
	connection send: 'world.checkpoint.save'.

! !

!Minecraft methodsFor: 'events' stamp: 'bf 2/25/2013 00:01'!
clearEvents
	connection send: 'events.clear'.
! !

!Minecraft methodsFor: 'initialize-release' stamp: 'bf 2/24/2013 22:06'!
close
	connection close
! !

!Minecraft methodsFor: 'initialize-release' stamp: 'bf 2/24/2013 21:17'!
connection: aConnection
	connection := aConnection.
! !

!Minecraft methodsFor: 'blocks' stamp: 'bf 2/25/2013 00:53'!
heightAt: vector2
	"return world height at xz-pos"
	^(connection send: 'world.getHeight' args: vector2 floor; receive) asNumber
! !

!Minecraft methodsFor: 'entities' stamp: 'bf 2/24/2013 22:33'!
playerIds
	"Get entity ids of all players"
	^self toNumbers: (connection send: 'world.getPlayerIds'; receive)
	! !

!Minecraft methodsFor: 'player' stamp: 'bf 2/24/2013 22:22'!
playerPos
	^self toNumbers: (connection send: 'player.getPos'; receive)
	! !

!Minecraft methodsFor: 'player' stamp: 'bf 2/24/2013 22:02'!
playerPos: vector3
	^connection send: 'player.setPos' args: vector3	! !

!Minecraft methodsFor: 'settings' stamp: 'bf 2/24/2013 23:54'!
playerSetting: key value: value
	"Keys:
		'autojump'
	"
	connection send: 'player.setting' args: {key. value}! !

!Minecraft methodsFor: 'player' stamp: 'bf 2/24/2013 22:22'!
playerTile
	^self toNumbers: (connection send: 'player.getTile'; receive)
	! !

!Minecraft methodsFor: 'player' stamp: 'bf 2/24/2013 22:02'!
playerTile: vector3
	^connection send: 'player.setTile' args: vector3	! !

!Minecraft methodsFor: 'events' stamp: 'bf 2/25/2013 00:32'!
pollBlockHits
	"triggered by sword right-click"
	^((connection send: 'events.block.hits'; receive)
		findTokens: '|')
			collect: [:event | self toNumbers: event]! !

!Minecraft methodsFor: 'entities' stamp: 'bf 2/24/2013 22:23'!
posOf: entityId
	^self toNumbers: (connection send: 'entity.getPos' args: entityId; receive)
	! !

!Minecraft methodsFor: 'entities' stamp: 'bf 2/24/2013 22:30'!
setPosOf: entityId to: vector3
	connection send: 'entity.setPos' args: {entityId}, vector3.

	! !

!Minecraft methodsFor: 'entities' stamp: 'bf 2/24/2013 22:30'!
setTileOf: entityId to: vector3
	connection send: 'entity.setTile' args: {entityId}, vector3.

	! !

!Minecraft methodsFor: 'entities' stamp: 'bf 2/24/2013 22:24'!
tileOf: entityId
	^self toNumbers: (connection send: 'entity.getTile' args: entityId; receive)
	! !

!Minecraft methodsFor: 'private' stamp: 'bf 2/24/2013 22:58'!
toArray: numberOrArray
	^numberOrArray isNumber
		ifTrue: [{numberOrArray}]
		ifFalse: [numberOrArray].! !

!Minecraft methodsFor: 'private' stamp: 'bf 3/10/2013 13:32'!
toNumbers: aString
	^Array streamContents: [:out | | in |
		in := aString readStream.
		[in atEnd] whileFalse: [
			out nextPut: (in upToAnyOf: ',|') asNumber]]
! !

!Minecraft methodsFor: 'settings' stamp: 'bf 2/24/2013 23:52'!
worldSetting: key value: value
	"Keys:
		'nametags_visible'
		'world_immutable'
	"
	connection send: 'world.setting' args: {key. value}! !

Object subclass: #MinecraftConnection
	instanceVariableNames: 'stream lastSent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Minecraft-Pi-Base'!

!MinecraftConnection commentStamp: 'bf 2/24/2013 23:30' prior: 0!
An MinecraftConnection is used to send commands to and receive results from a Minecraft game.

Instance Variables
	stream:		<SocketStream>

Example
	| conn pos |
	conn := MinecraftConnection host: 'raspberrypi' port: 4711.
	conn send: 'chat.post' args: 'Hi from Squeak!!'.
	pos := conn send: 'player.getPos'; receive.
	Transcript show: 'Player pos: '; show: pos; cr.
	conn close.
!

!MinecraftConnection class methodsFor: 'instance creation' stamp: 'bf 2/24/2013 20:44'!
host: hostName port: portNumber
	| stream |
	stream := SocketStream openConnectionToHostNamed: hostName port: portNumber.
	stream timeout: 5.
	^self new stream: stream! !

!MinecraftConnection methodsFor: 'initialize-release' stamp: 'bf 2/24/2013 18:12'!
close
	stream close.
! !

!MinecraftConnection methodsFor: 'private' stamp: 'bf 2/24/2013 20:39'!
drain
	"Drain all incoming data"
	[stream isDataAvailable]
		whileTrue: [ | data |
			data := stream nextAvailable.
			Transcript show: 'Drained: ', data; cr.
			Transcript show: 'Last message: ', lastSent].
! !

!MinecraftConnection methodsFor: 'private' stamp: 'bf 2/24/2013 23:43'!
print: argument on: aStream
	"Format arguments: comma-separated collections"
	| first |
	argument isString ifTrue: [^aStream nextPutAll: argument].
	argument isNumber ifTrue: [^aStream print: argument].
	argument == false ifTrue: [^aStream nextPut: $0].
	argument == true ifTrue: [^aStream nextPut: $1].
	"accept anything that implements #do:"
	first := true.
	argument do: [:each |
		first ifFalse: [aStream nextPut: $,].
		self print: each on: aStream.
		first := false].
! !

!MinecraftConnection methodsFor: 'data' stamp: 'bf 2/24/2013 23:29'!
receive
	"receive a line"
	| line |
	line := stream flush; nextLineLf.
	"Transcript show: 'Received ', line; cr."
	line = 'Fail'
		ifTrue: [^self error: lastSent, ' failed'].
	^line! !

!MinecraftConnection methodsFor: 'data' stamp: 'bf 2/24/2013 21:47'!
send: function
	^self send: function args: #()! !

!MinecraftConnection methodsFor: 'data' stamp: 'bf 2/24/2013 23:29'!
send: function args: arguments
	"Send 'function(arg1,arg2,...,argN)' and a linefeed"
	| line |
	self drain.
	line := String streamContents: [:s |
		s nextPutAll: function.
		s nextPut: $(.
		self print: arguments on: s.
		s nextPut: $).
		s lf].
	stream nextPutAll: line.
	stream flush.
	lastSent := line allButLast.
	"Transcript show: 'Sent ', line allButLast; cr."
! !

!MinecraftConnection methodsFor: 'initialize-release' stamp: 'bf 2/24/2013 16:10'!
stream: aSocketStream
	stream := aSocketStream.
! !
